"""
This project uses the Semantic Versioning scheme in conjunction with PEP 0440:
    <http://semver.org/>
    <https://www.python.org/dev/peps/pep-0440>

Major versions introduce significant changes to the API, and backwards
compatibility is not guaranteed. Minor versions are for new features and other
backwards-compatible changes to the API. Patch versions are for bug fixes and
internal code changes that do not affect the API. Development versions are
incomplete states of a release .

Version 0.x should be considered a development version with an unstable API,
and backwards compatibility is not guaranteed for minor versions.
"""

__version__ = "1.0"



from collections import OrderedDict
from django.contrib import admin
from django.db import models
from django.contrib.admin.utils import model_format_dict
from .forms import OptionAttrsActionForm


class GroupedActionsModelAdminMixin:
    """
    _summary_
    """

    action_form = OptionAttrsActionForm

    def get_action_choices(self, request, default_choices=models.BLANK_CHOICE_DASH):
        choices = [] + default_choices
        groups = OrderedDict()
        for func, name, description in self.get_actions(request).values():
            choice = (name, description % model_format_dict(self.opts))
            group_name = self._get_action_group(func)
            if group_name:
                if not group_name in groups:
                    groups[group_name] = []
                groups[group_name].append(choice)
            else:
                choices.append(choice)
        if groups:
            choices.extend(groups.items())
        return choices

    def _get_action_option_attrs(self, actions):
        extra_attrs = {}
        for func, name, _ in actions.values():
            description = self._get_action_verbose_description(func)
            if description:
                extra_attrs[name] = {'title': description}
        return extra_attrs

    def get_actions(self, request):
        actions = super().get_actions(request)

        option_attrs = self._get_action_option_attrs(actions)
        self.action_form.base_fields['action'].widget.option_attrs = option_attrs
        return actions

    @staticmethod
    def _get_action_verbose_description(func):
        return getattr(func, "description", func.__doc__ or None)

    @staticmethod
    def _get_action_group(func):
        return getattr(func, "group", None)


class GroupedActionsModelAdmin(GroupedActionsModelAdminMixin, admin.ModelAdmin):
    """
    _summary_
    """





class ActionGroup:
    def __init__(self, name, permissions=None):
        self.name = name
        self.permissions = permissions

    def __call__(self, func):
        func.group = self.name
        if self.permissions:
            action_perms = getattr(func, 'allowed_permissions', [])
            func.allowed_permissions = list(set(action_perms + self.permissions))
        return func


from django.contrib.admin.helpers import ActionForm
from django.utils.translation import gettext_lazy as _
from django import forms


class OptionAttrsSelect(forms.widgets.Select):
    """
    Select that allows to pass option-specific extra-attributes.
    """
    def __init__(self, attrs=None, option_attrs=None, choices=()):
        super().__init__(attrs, choices)
        self.option_attrs = option_attrs or dict()

    def create_option(self, *args, **kwargs):
        option = super().create_option(*args, **kwargs)
        option['attrs'].update(self.option_attrs.get(option['value'], dict()))
        return option


class OptionAttrsActionForm(ActionForm):
    """
    _summary_
    """
    action = forms.ChoiceField(label=_("Action:"), widget=OptionAttrsSelect())
